#!/usr/bin/perl
use strict;
use Bio::SeqIO;														#Handler for SeqIO Formats
use Bio::Tools::Run::StandAloneBlast; 		#Run Psi-Blast y Blastall
use Bio::Index::Fasta;                		#Index Fasta files

#Coded by Pablo Mier on 2015
#This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
#This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

my $time_start = time; #Start time
my $random = int(rand(1000000000000000));	#A random number is created, to identify each execution
my $upload_dir = "./results";		my $random_dir = "$upload_dir/$random";
`mkdir $random_dir`;	`chmod 777 $random_dir`;
my $file = ''; my $database = ''; my $filter = ''; my $eval = '';	my $ident = '';

##GET INPUT FILE##
for (my $i = 0; $i <= $#ARGV; $i++){
  if ($ARGV[$i] eq '-file'){   		$file = $ARGV[$i+1]; }				#Get initial query protein file
  if ($ARGV[$i] eq '-database'){  $database = $ARGV[$i+1]; }		#Get database
  if ($ARGV[$i] eq '-filter'){    $filter = $ARGV[$i+1]; }			#Get low complexity filter setting
  if ($ARGV[$i] eq '-evalue'){    $eval = $ARGV[$i+1]; }				#Get evalue setting
  if ($ARGV[$i] eq '-identity'){  $ident = $ARGV[$i+1]; 	}			#Get %id setting for the clustering of the results
}

if ($file eq '')	{  die "\nA valid filename is needed to start CABRA.\nThe correct usage of CABRA is \"perl CABRA.pl -file FILE\"\n\n"; }	#Die if user didn't provide a file
if ($database eq '') {	#SwissProt used as database as default
	$database = "./database/swissprot_2015_10.fasta";	
	print	"As you didn't provide any database, CABRA will use the default one (SwissProt version 2015_10).\n";
}	else {		#Format new database
		print "\nFormatting the selected database...\n\n";
		`makeblastdb -dbtype prot -parse_seqids -in $database`;
}		
if ($filter eq '') {					#LowComplexityFilter is OFF by default
	$filter = "F"; 	
	print "As you didn't select the LowComplexityFilter option, it will be OFF by default.\n";
}	
if ($eval eq '') { 						#Evalue is 0.00001 by default
	$eval = "0.00001";	
	print	"As you didn't provide any evalue cutoff for the BLAST search, CABRA will use the default one (evalue = 0.00001).\n";
}		

if ($ident eq '') { 					#Minimum %identity for the clustering is 53% by default
	$ident = 53;	
	print	"As you didn't provide any \%identity cutoff for the clustering, CABRA will use the default one (\%Id = 53%).\n";
} elsif (($ident < 20) || ($ident > 100)) {
	$ident = 20;	
	print	"The \%identity cutoff for the clustering must be between 20 and 100. As your selection is out of the range, CABRA will use the default one (\%Id = 53%).\n";
}

my $multifasta = Bio::SeqIO->new( -file  => "<$file" , '-format' => 'fasta');	#Multifasta file turns into a BioPerl object ($multifasta)
my $seq = $multifasta->next_seq();

##RUN BLAST##
my @paramet = (program => 'blastp',
							o => "$random_dir/output_blast.txt",
							e => $eval,
							F => $filter,		
							database => $database);	
my $blast = Bio::Tools::Run::StandAloneBlast->new(@{paramet});	#A BioPerl object is created, regarding Blastp parameters
my $report_blast = $blast->blastall($seq);

##SAVE RESULTS IN FASTA FILE##
my ($fasta,$results_identities) = &check_result($database, $report_blast);

if (length($results_identities) < 3) {
	unlink "formatdb.log";
	print "\nDear user, your BLAST search didn't get any results. Please, try again with another database. You can find the resulting BLAST report in: \"$random_dir/output_blast.txt\".\n\n";
} else {
	&fastaherder2($fasta,$random,$ident);		##RUN FASTAHERDER2, TO CLUSTER RESULTS##
	`mv ./$fasta ./results/$random/.`;							#Move multifasta file to result directory
	unlink "formatdb.log";
	print "\nDear user, your results are in the following folder: \"$random_dir\". The output is:\n";
	print "\toutput_blast.txt (BLAST raw output)\n";
	print "\tblast_result_fastas.txt (fasta file with the sequences from all the results from the BLAST output)\n";
	print "\t$random.cluster (HIGH confidence clusters)\n";
	print "\t$random.recluster (MEDIUM confidence clusters)\n";
	print "\t$random.leader (cluster representatives)\n\n";
}


exit;

###############################
##########SUBROUTINES##########
###############################
################################################################################################
sub fastaherder2 () {	
	my ($initial_file, $random,$ident) = @_;	
	my $r = 1; 	#Cluster numerical identification
	my $upload_dir = "./results";
	my $random_dir = "$upload_dir/$random";
	my $file_tempdir = "$upload_dir/$random";
	my $prim = 0;
	my $no_line = 0;
	my @fasta;

#######################################################################
###################### 1) fh_process_fasta.pl #########################
#######################################################################
### Input:	- Fasta file (mandatory) ##################################
### Output: - Table with lengths & identifiers ($output, .new) ########
###         - Chopped fasta files ($fastadir/from_X_to_Y) #############
#######################################################################

#GET & SORT INPUT DATA
my $prev;
my $step = 25; #Step is 25 amino acids by default. The user can change it
my $file = $initial_file;

if ($file eq ''){  die "A valid filename is needed to start FastaHerder_v2. Type -help for more info\n\n"; }	#Die if user didn't provide a file

my $fastadir = $random_dir."/fastas/";	#Create temporal directory
system(`mkdir $fastadir`);			system(`chmod -R a+w $fastadir`);

my $new = $file_tempdir."/".$random.".fasta";
my $fasta = $file_tempdir."/".$random.".fas"; 		#Fasta file with LCR OFF (default)
my $output = $fasta.".matrix";
my $half_diff_threshold = 32; 	

#READ INITIAL FILE AND CHOP IT DEPENDING ON THE SEQUENCE LENGTH
my @fields;	my $ide;	my $seq;	my $length;		my $min = 0; my $max; my $namedir;
my $header = 1;	my %proteins; my $dir; my $dir_sup;	my $OUT_FA;
my $nn_seq = 1;

open(FASTA,$file) || die "cannot open $file file\n";

while(<FASTA>) {
  chomp $_;
  if ($header >= 1){
    if ($_ =~ /^\>/) {
      @fields = split(/\s+/,$_);	
      $ide = $fields[0];
      $header--;
      next;
    }
	} elsif ($_ =~ /^\>/){
			$nn_seq++;
		  @fields = split(/\s+/,$_);   
			($length,$ide,$max) = &id($seq,$ide,$step,$dir,$fastadir,$new,$max);
			$proteins{$length} .= $ide."|*|";					#Record info
		  $seq = '';
		  $ide = $fields[0];
		  next;
		} elsif ($_ =~ /[a-zA-Z]/){
      $seq .= $_;  
  }
}
close(FASTA);

($length,$ide,$max) = &id($seq,$ide,$step,$dir,$fastadir,$new,$max);
$proteins{$length} .= $ide."|*|";					#Record info
open(OUTFILE,">>$output");
my $number = 1;
for (my $i = $max; $i >= 0; $i-- ){
	my @a = '';
	if (exists $proteins{$i}){
		@a = split(/\|\*\|/,$proteins{$i});
		foreach my $item(@a){
		  print OUTFILE "$i\t$item\t$i\t$number\n";
		  $number++;
		}
  }
}
close(OUTFILE);

#######################################################################
####################### 2) fh_neocluster.pl (1:) ######################
### Input:  - Table with lengths & identifiers ($output, .new) ########
###         - Chopped fasta files ($fastadir/from_X_to_Y) #############
### Output:	- Table with the clusters ($clusters, .cluster) ##########
#######################################################################
#DEFINE INITIAL VALUES
my $tol = 0;			my $ll = "no";		my $lccr = "F";

my $do_continue = 0;
my $tempfile = $file_tempdir."/temp";
my $topfile = $file_tempdir."/top";
my $blastout = $file_tempdir."/blastout";
my $WK0 = $file_tempdir."/WK0";
my $WK1 = $file_tempdir."/WK1";
system(`cp $output $WK0`);	#Copy the input to WK0

#MAKES BUFFER & WRITES TEMPFILE WITH LEFTS
open(WK0,"<$WK0");
open(TEMP,">>$tempfile");

my $cont_buffer = 0;			my $minimum = 0;					my $k = 0;			my $top = 1;		#Clean buffer
my $current_length = 0;		my $current_seq = '';			my %buffer = '';								#Clean buffer
my $limit_inf;						

while(<WK0>){	
	@fields = '';
  chomp $_;
  @fields = split(/\t/,$_);
  $length = $fields[0];		#Instead of looking to the seq length we look to the column that has the length of shorter sequence in its cluster
  $ide = $fields[1];
  $minimum = $fields[2];
 	$limit_inf = '';

  if ($top == 1){					#The length of the top is still taken from the left
    $current_length = $fields[0];
    $current_seq = $ide;
		($half_diff_threshold) = &len($current_length);
    $top--;
    $limit_inf = $current_length - $half_diff_threshold;
  }
  if (($minimum >= $limit_inf) && ($minimum <= $current_length)){
      $buffer{$ide} = $length;   
      $cont_buffer++;
  } else {
  		print TEMP "$_\n";
  	  $k++;
  }
}
close(WK0);
close(TEMP);

system(`cp $tempfile $WK0`);	#New WK0 with left sequences
my $left_sequences = $k;

#FIRST ITERATION
my %yyya;

my $max_length; my $q_aln_length; my $s_aln_length;
my %sequence; my $see; my $report_blast;
my $cont_flusher = 0;
my $do_read;	my %present;
my $clusters = $output."cluster";
if ($cont_buffer > 1){
	$do_read = 1;
	my $current_length; my $current_seq; my $query; my @q; my %sequence; my $dir; my $dir_sup; my $QUERY_FA;  my $top = 1; my $l;
	if ($do_read == 1){		open(WK1,">$WK1");	}
	foreach my $seq(sort { $buffer{$b} <=> $buffer{$a} } keys %buffer){    #Wite TOP file for BLAST
		$l = $buffer{$seq};
		next if ($l < 1);
		if ($top == 1){
			$current_length = $l;
			$current_seq = $seq;
			$top--;
		  if (exists $sequence{$seq}){
				$q[0] = $seq;
				$q[1] = $sequence{$seq};
		  } else {
					$dir = $step * int($l/$step);
					$dir_sup = $dir + $step;
					$QUERY_FA = $file_tempdir."/fastas/from_".$dir."_to_".$dir_sup."/".$l.".fa";
					$query = '';
					$query = `grep ^"$seq" $QUERY_FA`;
					@q = split(/\t/,$query);
			}
			open (TOP,">$topfile");
			next if (length ($q[1]) < 1);
			print TOP "$q[0]\n$q[1]";
			close TOP;
			last;		
		}  
	}
	($half_diff_threshold) = &len($current_length);
	if ($do_read == 1){	#Get the sequences
		for ($l = $current_length; $l >= $limit_inf; $l--){
			$dir = $step * int($l/$step);  
		  $dir_sup = $dir + $step;
		  $QUERY_FA = $file_tempdir."/fastas/from_".$dir."_to_".$dir_sup."/".$l.".fa";
	
		  open(QUERY_FA,$QUERY_FA);
		  while(<QUERY_FA>){
				chomp $_;
				@q = split(/\t/,$_);
				$q[1] =~ s/ //g;

				if (exists $buffer{$q[0]}){				#Write fasta to WK1
					$sequence{$q[0]} = $q[1];
					next if ((length ($q[0]) < 1) || (length ($q[1]) < 1));
					print WK1 "$q[0]\n$q[1]\n";
				}
		  }
			close QUERY_FA;
		}
	}
	close WK1;
	$cont_flusher = 0;
	`chmod 777 $WK1`;
	unlink "$WK1.idx"; unlink "$WK1.phr"; unlink "$WK1.psq"; unlink "$WK1.pin";
	my $index = Bio::Index::Fasta->new (-filename => $WK1 . ".idx", -write_flag => 1);
	$index->make_index($WK1);
	&run_formatdb("T", "$WK1");

	my $report_blast = &blast($WK1,$blastout,$lccr,$topfile,$random);
	$cont_flusher++;	
	my $valid; my $id; my $clustered_string;
	while (my $result = $report_blast->next_result) {  	#Check every result
		while (my $hit = $result->next_hit) {  						#Check all hits
			($valid,$id) = &parse($hit,$half_diff_threshold,$max_length,$current_length,$ident);
			if ($valid == 1){
				if ( exists $present{$id} ){
				} else {
						$clustered_string .= $id.';';
						$present{$id} = 1;
					}
			}
		}
	} 

  $see = ">".$id;							#Remove validated sequence from the buffer
	delete $buffer{$see};
	open(CLUSTERS,">>$clusters");   #Write down cluster
  my @cstring = split(/\;/,$clustered_string);
	my $ne = $current_seq;
	$ne =~ s/>/>$r-(1:)/;
  my $see = $current_seq;
  $see =~ s/^>//;
  
	if (exists $yyya{$see}) { }
	else {	
		print CLUSTERS "$ne-"; $r++;	$yyya{$see} = 1; $present{$see} = 1; 
	 	foreach my $item(@cstring){
		 	if ($item ne $see){
				if (exists $yyya{$item}) { }
				else {	print CLUSTERS "(1:)$item;";	$yyya{$item} = 1; }		  
	 		}
 	 }
		print CLUSTERS "\n";  close CLUSTERS;
	}		 
  
  delete $buffer{$current_seq};    delete $sequence{$current_seq};

	my $do_read = 1;  my $top = 1;  my $cont_buffer = 0;
  my $saved_length = $current_length;
  my $current_seq = '';  my $current_length = '';

  foreach my $seq (sort { $buffer{$b} <=> $buffer{$a} } keys %buffer){
		my $l = $buffer{$seq};
		if ($top == 1){
	    $current_length = $l;
	    $current_seq = $seq;
	    if ($current_length eq $saved_length ){	    #If current length has not decreased it doesn't need to read more sequences
				$do_read = 0;
	    }
	    $top--;
		} 
		$cont_buffer++;
	}
  if ($cont_flusher > 999) {
		$do_read = 1;
  }

} else {		#If buffer is a singleton
		open(CLUSTERS,">>$clusters");
		my $ne = $current_seq;
		
		$ne =~ s/>/>$r-(1:)/;
		my $see = $current_seq;
		$see =~ s/^>//;
		if (exists $yyya{$see}) { }
		else {	print CLUSTERS "$ne-"; $r++;	$yyya{$see} = 1; $present{$see} = 1; }		  

    close(CLUSTERS);
    $cont_buffer = 0;    $current_seq = '';    $current_length = '';    $do_read = 0;    #Cleans buffer
}

#MAIN LOOP
until (($left_sequences <= 0 ) && ($cont_buffer <= 1)){
	if ($current_seq ne ''){
		($half_diff_threshold) = &len($current_length);
	}
  if ($do_read == 1){
 		open(WK0,$WK0) || die "cannot open $WK0 file\n";
		open(TEMP,">>$tempfile");
		$top = 1;		$k = 0;
		while(<WK0>){
	    chomp $_;
	    @fields = split(/\t/,$_);
	    $length = $fields[0];
	    $ide = $fields[1];
	    $minimum = $fields[2];
	    #in case buffer is empty get new top seq from WK0
	    if ($current_seq eq '') {
				if ($top == 1){
					$current_length = $fields[0];
					$current_seq = $ide;
					$top--;		
					($half_diff_threshold) = &len($current_length);
				}
	    }
	    my $limit_inf = $current_length - $half_diff_threshold;
	    if (($minimum >= $limit_inf) && ($minimum <= $current_length)) {	#Change the condition for the smallest in the cluster
	    	$buffer{$ide} = $length;
				$cont_buffer++;
	    } else {
					print TEMP "$_\n";
					$k++;
	    }
		}
		close WK0;		close TEMP;
    
		system(`cp $tempfile $WK0`);			#The new WK0 with left sequences
		$left_sequences = $k;
	}

	if ($cont_buffer > 1){		#Operations on buffered sequences
		my $current_length; my $current_seq; my $query; my @q; my %sequence; my $dir; my $dir_sup; my $QUERY_FA;  my $top = 1; my $l;
		if ($do_read == 1){
			open(WK1,">$WK1");
		}

		foreach my $seq(sort { $buffer{$b} <=> $buffer{$a} } keys %buffer){    #Wite TOP file for BLAST
			$l = $buffer{$seq};
			next if ($l < 1);
			if ($top == 1){
				$current_length = $l;
				$current_seq = $seq;
				$top--;
				if (exists $sequence{$seq}){
					$q[0] = $seq;
					$q[1] = $sequence{$seq};
				} else {
						$dir = $step * int($l/$step);
						$dir_sup = $dir + $step;
		 
						$QUERY_FA = $file_tempdir."/fastas/from_".$dir."_to_".$dir_sup."/".$l.".fa";
						$query = '';
						$query = `grep ^"$seq" $QUERY_FA`;
						@q = split(/\t/,$query);
				}
				open (TOP,">$topfile");
				next if ((length ($q[0]) < 1) || (length ($q[1]) < 1));
				print TOP "$q[0]\n$q[1]\n";
				close TOP;
				last;		
			}  
		}
		  
		if ($do_read == 1){	#Get the sequences
			for ($l = $current_length; $l >= $limit_inf; $l--){
				$dir = $step * int($l/$step);  
				$dir_sup = $dir + $step;
				$QUERY_FA = $file_tempdir."/fastas/from_".$dir."_to_".$dir_sup."/".$l.".fa";
	
				open(QUERY_FA,$QUERY_FA);
				while(<QUERY_FA>){
					chomp $_;
					@q = split(/\t/,$_);
					$q[1] =~ s/ //g;

					if (exists $buffer{$q[0]}){				#Write fasta to WK1
						$sequence{$q[0]} = $q[1];
						next if ((length ($q[0]) < 1) || (length ($q[1]) < 1));
						print WK1 "$q[0]\n$q[1]\n";
					}
				}
				close QUERY_FA;
			}
		}
		close WK1;

		my $size2 = (-s $WK1);
		if ($size2 > 0) {
			if ($do_read == 1){			#Writes db & formats only if current length has decreased
				unlink "$WK1.idx"; unlink "$WK1.phr"; unlink "$WK1.psq"; unlink "$WK1.pin";
				my $index = Bio::Index::Fasta->new (-filename => $WK1 . ".idx", -write_flag => 1);
				$index->make_index($WK1);
				&run_formatdb("T", "$WK1");
				#`makeblastdb -dbtype prot -in $WK1`;					#Index database	
				$cont_flusher = 0;
			}
			my $multi2 = Bio::SeqIO->new( -file  => "<$topfile" , '-format' => 'fasta');	#Get the query
			my $ttp = $multi2->next_seq();
			my $report_blast = &blast($WK1,$blastout,$lccr,$ttp,$random);
			$cont_flusher++;	
			my $valid; my $id; my $clustered_string;
			while (my $result = $report_blast->next_result) {  	#Check every result
				while (my $hit = $result->next_hit) {  					#Check all hits
					($valid,$id) = &parse($hit,$half_diff_threshold,$max_length,$current_length,$ident);
					if ($valid == 1){
						if ( exists $present{$id} ){
						} else {
							$clustered_string .= $id.';';
							$present{$id} = 1;
						}
					}
				 }
			} 
			$see = ">".$id;							#Remove validated sequence from the buffer
			delete $buffer{$see};

			open(CLUSTERS,">>$clusters");   #Write down cluster
			my @cstring = split(/\;/,$clustered_string);
			my $ne = $current_seq;
			$ne =~ s/>/>$r-(1:)/;
			my $see = $current_seq;
			$see =~ s/^>//;

			if (exists $yyya{$see}) { }
			else {	
				print CLUSTERS "$ne-"; $r++;	$yyya{$see} = 1; $present{$see} = 1; 
				foreach my $item(@cstring){
				 	if ($item ne $see){
						if (exists $yyya{$item}) { }
						else {	print CLUSTERS "(1:)$item;";	$yyya{$item} = 1; }		  
			 		}
				}
				print CLUSTERS "\n";								close CLUSTERS;
			}	   

			delete $buffer{$current_seq};  			delete $sequence{$current_seq};

			my $do_read = 1;  my $top = 1; my $cont_buffer = 0;
			my $saved_length = $current_length;
			my $current_seq = '';  my $current_length = '';

			foreach my $seq (sort { $buffer{$b} <=> $buffer{$a} } keys %buffer){
				my $l = $buffer{$seq};
				if ($top == 1){
					$current_length = $l;
					$current_seq = $seq;
					if ($current_length eq $saved_length ){	    #If current length has not decreased it doesn't need to read more sequences
						$do_read = 0;
					}
					$top--;
				} 
				$cont_buffer++;
			}
			if ($cont_flusher > 999) {
				$do_read = 1;
			}
		}
	} else {    #If buffer is a singleton
			open (CLUSTERS,">>$clusters");
			my $ne = $current_seq;
			$ne =~ s/>/>$r-(1:)/;
			my $see = $current_seq;
			$see =~ s/^>//;
			if (exists $yyya{$see}) { }
			else {	print CLUSTERS "$ne-"; $r++;	$yyya{$see} = 1; $present{$see} = 1; }	  

			$cont_buffer = 0;		$current_seq = '';		$current_length = '';		$do_read = 1;		#Cleans buffer
  }
	$cont_buffer--;
}

#APPENDS THE REST OF UNCLUSTERED SEQUENCES LEFT IN WK1
open(CLUSTERS,">>$clusters");
open(WK1,$WK1);
while(<WK1>){
	chomp $_;
	next if (length($_)<1);
  if ($_ =~ /^>/) {
		$_ =~ s/^>//;
		if (exists $present{$_}){
		}else{
			next if (exists $yyya{$_});
	    print CLUSTERS ">$_\n";
	    $present{$_} = 1;
		}
	}
}
close WK1;	close CLUSTERS;

my %buffer2;
my $mul2 = Bio::SeqIO->new( -file  => "<$new" , '-format' => 'fasta');	
while (my $ss2 = $mul2->next_seq()) {  	#Go over the multifasta file, each sequence at a time
	my $na2 = $ss2->display_id;						#Get its identifier
	my $seeq2 = $ss2->seq;								#Homologue's sequence
	$buffer2{$na2} = $seeq2;
}

#######################################################################
######################### 3) fh_leaders.fasta.pl ######################
### Input:  - File with the clusters ($clusters, .cluster) ###########
### Output:	- File with the leaders ($lead, .leaders.fasta) ###########
#######################################################################
#PREPARE FASTA WITH LEADERS
my $lead = $file_tempdir."/$random".".leader";
#system(`chmod a+w $lead`);
my %leaders = '';	#Leaders' identifiers
my $print_line;

my $otr = "$clusters-2";
open(NEE,">>$otr") || die;
open(CLUSTERS,$clusters) || die;
while(<CLUSTERS>) {		#Read clusters & get leaders ID
	chomp $_;
  if ($_ =~ /^\>/) {
		print NEE "$_\n";
		my @fields = split(/-/,$_);	
		$ide = $fields[0]."-".$fields[1];
		$leaders{$ide} = 1;
		next;
  }
}
close CLUSTERS; close NEE; 

open(OUTPUT,">$lead");
my $nn_cl = 0;						#Number of clusters
foreach my $sq(sort keys %leaders){    
	next if (length ($sq) < 1);
	my @fie = split(/\)/,$sq);	
	print OUTPUT "$sq\n$buffer2{$fie[1]}\n";
	$nn_cl++;
}
close OUTPUT;
system(`cp $otr $clusters`);	

unlink $otr; unlink "$WK1.phr";	unlink "$WK1.pin";	unlink "$WK1.psq";	unlink $blastout;	unlink $tempfile;	unlink $topfile;	unlink $WK0;	unlink $WK1;

#######################################################################
########################### 4) re-cluster (2:) ########################
### Input:  - File with the clusters ($clusters, .cluster) ############
### 				- File with the leaders ($lead, .leaders.fasta) ###########
### Output: - File with the reclusters ($recluster, .recluster) #######
#######################################################################
my %recluster;	my @extra_array; my @que_id;	my %final; my %final2;
my $co=0;				my $recluster = $output."recluster";

my $mul = Bio::SeqIO->new( -file  => "<$lead" , '-format' => 'fasta');	#Multifasta file
while (my $ss = $mul->next_seq()) {  	#Go over the multifasta file, each sequence at a time
	push(@extra_array,$ss);	
	my $na = $ss->display_id;						#Get its identifier
	push(@que_id,$na);									#Save the sequence identifier within the others 
	my $seeq = $ss->seq;								#Homologue's sequence
}
my $number_extra = $#extra_array + 1;		#Number of query proteins

unlink "$lead.idx"; unlink "$lead.phr"; unlink "$lead.psq"; unlink "$lead.pin";
my $index = Bio::Index::Fasta->new (-filename => $lead . ".idx", -write_flag => 1);
$index->make_index($lead);
&run_formatdb("T", "$lead");

foreach my $cl(sort keys %buffer2){    #Go over all initial sequences	
	my $ya = 0;
	open(CLUSTERS,$clusters);
	while(<CLUSTERS>) {		
		chomp $_;
		if (index($_, $cl) != -1) {			#If the cluster contains the sequence
			$ya = 1;
		} 
	}
	close CLUSTERS;

	if ($ya == 0) {		#Those sequences that were not clustered, can be reclustered with a low threshold for length (2:)
		$co++;
		my $aux = $clusters."aux";
		open(FUERA,">>$aux");
		print FUERA ">$cl\n$buffer2{$cl}";
		close FUERA;
		my $multi2 = Bio::SeqIO->new( -file  => "<$aux" , '-format' => 'fasta');	#Get the query
		my $ttp = $multi2->next_seq();
		unlink $aux;
		my $report_blast = &blast($lead,$blastout,$lccr,$ttp,$random);	
		my $result = $report_blast->next_result;  		#Check the first result
		if ($result->num_hits > 0) {									#If there is at least one result
			my $hit = $result->next_hit;  							#Check the first hit
			my $res_id = $hit->name;
			my $hsp = $hit->next_hsp;
			my $id_hsp = $hsp->frac_identical;	#Hsp's %identity 
			my $threshold = 0.53;								#Minimum identity theshold (orthoFinder)

			if ($ident > 0) {		$threshold = $ident/100;	}	

			if ($id_hsp >= $threshold) {					#Hsp's identity must be greater than the estimated threshold
				$recluster{$cl} = $res_id;					#Relate query's id and first hit's id
			} else {
				$recluster{$cl} = "____________";		#Value of extra coclusters
			}
		} else {																#If the extra query is a singleton
				$recluster{$cl} = "____________";		#Value of extra coclusters
		}
	}
}

if ($co > 0) {
	my @clus;
	open(CLUSTERS,$clusters) || die;
	while(<CLUSTERS>) {		#Read clusters
		chomp $_;
		push(@clus,$_);
	}
	close CLUSTERS;
	my $number_clus = $#clus + 1;		#Number of clusters

	my $ok = 0;
	foreach my $cocl(sort keys %recluster){    #Go over all reclusters
		for (my $a = 0; $a < $number_clus; $a++){
			if (index($clus[$a], $recluster{$cocl}) != -1) {
				$clus[$a] = $clus[$a]."(2:)$cocl;";
			}
		}	
	}

	open(LAST,">>$recluster") || die;
	for (my $a = 0; $a < $number_clus; $a++){
		print LAST "$clus[$a]\n";
	}
	close LAST;
}

#######################################################################
########################### 5) Write all sequences ####################
### Input:  - File with the clusters ($clusters, .cluster) ############
### 			  - File with the reclusters ($recluster, .recluster) #######
#######################################################################
#Writes all of the initial sequences that haven't been written yet
my $nn = 0;
foreach my $cl(sort keys %buffer2){    #Go over all initial sequences	
	my $ya = 0;
	open(CLUSTERS,$recluster);
	while(<CLUSTERS>) {		
		chomp $_;
  	if (index($_, $cl) != -1) {
			$ya = 1;
		}
	}
	close CLUSTERS;
	if ($ya == 0) {
		open(RE,">>$recluster");
		print RE ">$r-(1:)$cl\n";
		$r++;
		close RE;
	}
}

unlink "$lead.idx";	unlink "$lead.phr";	unlink "$lead.pin";	unlink "$lead.psq";	
unlink $new; unlink $blastout; unlink $output; 
`rm -r -f "$file_tempdir/fastas/"`;
unlink "$random_dir/$random.0fasta";
`mv "$random_dir/$random.fas.matrixcluster" "$random_dir/$random.cluster"`;
`mv "$random_dir/$random.fas.matrixrecluster" "$random_dir/$random.recluster"`;
unlink "$WK1.idx";

}
################################################################################################
sub check_result () {	
	my ($database, $report_blast) = @_;	
	my $outfile = "blast_result_fastas.txt";
	open (OUT, ">>$outfile");
	my $ii = '';
	while (my $result = $report_blast->next_result) {
		while (my $hit = $result->next_hit) {
			my $acc = $hit->name;		
			my $prot = `blastdbcmd -db $database -dbtype prot -entry '$acc'`;		#Get the candidate sequence
			print OUT "$prot";
			my $hsp = $hit->next_hsp;
			my $sco; my $id;			
			if ($hsp) {
				$sco = $hsp->score;
				$id = $hsp->percent_identity;		$id = sprintf "%.2f", $id; 			$id = $id."%";	
				$ii .= "$acc:$sco:$id||";		#AC, score, %identity
			}
		}
	}
	close OUT;
return ($outfile,$ii);
}
################################################################################################
sub run_formatdb () {		
	my ($ft, $db) = @_;
	system("formatdb -p $ft -i $db");		#Run formatdb to $db file
	return;
}
################################################################################################
sub id () {
	my ($seq,$ide,$step,$dir,$fastadir,$new,$max) = @_;
	my $length = 0;	 my %proteins; my $min = 0; my $dir_sup;
	my $namedir = '';
	$seq =~ s/ //g;
	my @l = split(//,$seq);
	$length = scalar(@l);
	$proteins{$length} .= $ide."|*|";
	if ($length > $max){   $max = $length;  }
	if ($length < $min){   $min = $length;  }
	$dir = $step * int($length/$step); 
	$dir_sup = $dir + $step;
	$namedir = $fastadir."/from_".$dir."_to_".$dir_sup;
	if ($length >= 0){
		mkdir($namedir,0777) unless ( -d $namedir );
		my $OUT_FA = $namedir."/".$length.".fa";
		next if (length ($seq) < 1);
		open (NEW, ">>$new");						print NEW "$ide\n$seq\n";			close NEW; 	
		open (OUT,">>$OUT_FA");					print OUT "$ide\t$seq\n";			close OUT;
	}

	return ($length,$ide,$max);
}
################################################################################################
sub parse () {
	my ($hit,$half_diff_threshold,$max_length,$current_length,$ident) = @_;
	my $id = $hit->name;
	my $subject_length = $hit->length;
	next if ($subject_length < 1);
	my $hsp = $hit->next_hsp;
	my $identities = $hsp->percent_identity;
	my $q_begin = $hsp->start('query');
	my $q_end = $hsp->end('query');
	my $s_begin = $hsp->start('hit');
	my $s_end = $hsp->end('hit');
	my $valid = 1;	my $aln_length;
	my $diff_allowed = $half_diff_threshold/2;
	if ($q_begin > $diff_allowed){	$valid = 0;    } 
	elsif ($s_begin > $diff_allowed){	$valid = 0;  } 
	elsif ($q_end < $max_length - $diff_allowed) {	$valid = 0; } 
	elsif ($s_end < $subject_length - $diff_allowed){	$valid = 0;    } 
	else {	#Boundaries were OK
		my $q_aln_length = $q_end - $q_begin;	
		my $s_aln_length = $s_end - $s_begin;
		if ($q_aln_length > $s_aln_length){		$aln_length = $q_aln_length;	} 	#Take as alignment length the longest of the two
		else {	$aln_length = $s_aln_length;	}
		if ($aln_length < ($current_length * 0.5)) {	$valid = 0;	}		#Query coverage MUST be at least 50%
		if ($aln_length > 80){		#Apply HSSP threshold: Schneider & Sander Proteins 1991
			if ($identities < 24.8){			$valid = 0;			}
			if ($identities < $ident){		$valid = 0;		}
		} else {
			if ($identities < (290.15 * ($aln_length ** -0.562))){	$valid = 0;	}		
			if ($identities < $ident){		$valid = 0;		}
		}
	}
return ($valid,$id);
}
################################################################################################
sub len () {
	my ($current_length) = @_;
	my $half_diff_threshold = 32; 		my $half_diff_threshold200 = 20;  my $half_diff_threshold100 = 10; 	#Original parameters
	my $half_diff_threshold60 = 7; 		my $half_diff_threshold40 = 5; 		my $half_diff_threshold20 = 2; 		

	if ( $current_length < 200 ) {   $half_diff_threshold = $half_diff_threshold200;	}			
	if ( $current_length < 100 ) {   $half_diff_threshold = $half_diff_threshold100;	}
	if ( $current_length < 60 ) {    $half_diff_threshold = $half_diff_threshold60;		}			
	if ( $current_length < 40 ) {	   $half_diff_threshold = $half_diff_threshold40;		}
	if ( $current_length < 20 ) {    $half_diff_threshold = $half_diff_threshold20;		}
	return ($half_diff_threshold); 
}
################################################################################################
sub blast () {
	my ($db,$blastout,$lccr,$topfile,$random) = @_;
	my @paramet = (program => 'blastp',
		     				database => $db,	#Database
								F => $lccr,				#Low complexity filter
								e => 0.001);			#E-value threshold
	my $blast = Bio::Tools::Run::StandAloneBlast->new(@{paramet});	#A BioPerl object is created, regarding Blastp parameters
	my $report_blast = $blast->blastall($topfile);		#The Blastp is done
	return ($report_blast);
}
################################################################################################
